/* JavaPolicyEditor.java 
 * 
 * helper class to create/display/modify the .java.policy file.
 * Use 'make policyjar' to compile, jar, and sign the class.
 *
 * 30.01.06 - new class
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;


/**
 * simple tool to create, display, and modify the $HOME/.java.policy file
 * used to control applet security settings by the Java virtual machine.
 * <p>
 * This class provides a simple user-interface to display and modify the
 * contents of the "security policy" file used by the Java virtual machine
 * to control the security settings for (unsigned) applets.
 * The security mechanism built into the Java programming language and
 * the corresponding virtual machine allows each user selecting what 
 * operations are permitted or forbidden for applets downloaded from websites.
 * <p>
 * Naturally, unknown applets are denied all sensitive operations in order
 * to fulfil the safety and security guarantees of the Java environment.
 * Only entries specifically added to the policy file by the user give
 * the required rights to applets.
 * This application allows you granting the required permissions before
 * running the Hades simulation framework, as a few of the Hades applet
 * require extra permissions (e.g. AudioPermission to play sampled audio).
 * <p>
 * If you trust us, you can run the JavaPolicyEditor.jar archive directly
 * from our website - but you really shouldn't. Instead, download the source
 * code, read and study the source code, then compile the source code yourself.
 * Afterwards, run the newly compiled code to create a .java.policy file
 * (if you don't already have it) and modify it to add the required permissions.
 * Naturally, you can also directly edit the .java.policy file with your
 * favorite text editor (recommended) or the (terribly user-unfriendly)
 * tool called "policytool" provided  as part of the JDK/JRE.
 */
public class JavaPolicyEditor implements ActionListener {

  public final static int n_cols = 60;
  public final static int n_rows = 30;

  private JFrame       frame;
  private JLabel       infoLabel;
  private JTextArea    textArea;
  private JScrollPane  scroller;
  private JButton      rereadButton, hadesButton, saveButton, exitButton;



  public JavaPolicyEditor() {
    frame = new JFrame( "Java Applet Permissions Editor" );

    infoLabel = new JLabel( getJavaPolicyLocation() );
    textArea = new JTextArea( n_rows, n_cols );
    scroller = new JScrollPane( textArea );
    scroller.setVerticalScrollBarPolicy(
                   ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS );
    scroller.setViewportBorder( new EmptyBorder(5,5,5,5) );

    JPanel p = new JPanel();
    p.setLayout( new FlowLayout(FlowLayout.CENTER));

    rereadButton = _cb( "Read .java.policy",
                        "(Re-) read editor contents from .java.policy file" );
    hadesButton  = _cb( "Add Hades applet permissions",
                        "Add permissions required for Hades applet" );
    saveButton = _cb( "Save to .java.policy", 
                      "Save current editor contents to .java.policy file" );
    exitButton = _cb( "Exit",
                      "Exit the editor without saving the .java.policy file" );

    p.add( rereadButton );
    p.add( hadesButton );
    p.add( saveButton );
    p.add( exitButton );

    frame.getContentPane().setLayout( new BorderLayout() );
    frame.getContentPane().add( "North",  infoLabel );
    frame.getContentPane().add( "Center", scroller );
    frame.getContentPane().add( "South", p );
    frame.pack();
   
    frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE ); 
  }


  public JButton _cb( String label, String tooltip ) {
    JButton tmp = new JButton( label );
    tmp.setToolTipText( tooltip );
    tmp.addActionListener( this );
    return tmp;
  }


  public JFrame getJFrame() {
    return frame;
  }


  public void actionPerformed( ActionEvent evt ) {
    // msg( "-#- actionPerformed: " + evt );
    Object src = evt.getSource();

    if (src == rereadButton) {
      String contents = readJavaPolicyFile();
      textArea.setText( contents );
      textArea.setCaretPosition( contents.length() );
    }
    else if (src == hadesButton) {
      String contents = textArea.getText() 
                      + "\n"
                      + getHadesAppletPermissions();
      textArea.setText( contents );
      textArea.setCaretPosition( contents.length() );
    }
    else if (src == saveButton) {
      saveEditorContentsToPolicyFile();
    }
    else if (src == exitButton) {
      System.exit( 0 );
    }
    else {
      msg( "-E- unknown event source: " + src );
    }
  }


  public String getJavaPolicyLocation() {
    return System.getProperty( "user.home" )
           + File.separator 
           + ".java.policy";
  }


  public String readJavaPolicyFile() {
    String s = null;
    try {
      File   f = new File( getJavaPolicyLocation() );
      if (!f.exists()) return "";

      FileReader       fr = new FileReader( f );
      CharArrayWriter caw = new CharArrayWriter();
      int i;
      while ( (i = fr.read()) != -1) {
        caw.write( (byte) i );
      }
      fr.close();
      return caw.toString();
    }
    catch( Exception e ) {
      e.printStackTrace();
      return e.toString();
    }
  }


  /**
   * return the permissions required to run Hades with FreeTTS speech synthesis
   * as an applet. 
   * Obviously, we need AudioPermission to play any sound.
   * <p>
   * Granting the PropertyPermissions is harmless, and required for both
   * FreeTTS and Jython scripting. The most dangerous operation here is
   * finding out your home directory location via "user.home".
   * <p>
   * The ClassLoader stuff _is_ dangerous, and should be monitored carefully.
   * FreeTTS really shouldn't need those permission, but the convenience
   * mechanism used to find installed extra voices at runtime relies on access
   * to the classloader. Hacking FreeTTS might remove this dependency.
   * Jython scripting also needs to create its own class loader, but this
   * is essential due to the way that Jython operates (compiling its own
   * source files to classes at runtime).
   */
  public String getHadesAppletPermissions() {
return 
  "/* PERMISSIONS ADDED BY JavaPolicyEditor BEGIN */\n"
+ "grant codeBase \"http://tams-www.informatik.uni-hamburg.de/applets/hades/webdemos/-\" {\n"
+ "  permission javax.sound.sampled.AudioPermission \"play\";\n"
+ "  permission java.util.PropertyPermission \"*\", \"read,write\";\n"
+ "  permission java.lang.RuntimePermission \"createClassLoader\";\n"
+ "  permission java.lang.RuntimePermission \"getClassLoader\";\n"
+ "  permission java.lang.RuntimePermission \"getProtectionDomain\";\n"
+ "};\n"
+ "/* PERMISSIONS ADDED BY JavaPolicyEditor END */\n"
+ "\n"
;
  }


  public String _dd( int i ) {
    if (i < 0) return "X" + i + "X";
    if (i < 10) return "0" + i;
    return "" + i;
  }


  public void saveEditorContentsToPolicyFile() {
    try {
      String   s = getJavaPolicyLocation();
      Calendar c = Calendar.getInstance();
      String t = s 
                 + ".backup_" 
                 + _dd( c.get( Calendar.YEAR ))
                 + _dd( c.get( Calendar.MONTH ))
                 + _dd( c.get( Calendar.DAY_OF_MONTH ))
                 + "_"
                 + _dd( c.get( Calendar.HOUR ))
                 + _dd( c.get( Calendar.MINUTE ))
                 + _dd( c.get( Calendar.SECOND ))
                 + ".txt";

      msg( "-#- backup filename='" + t + "'" );

      // move the existing original .java.policy file to a backup location,
      // with name ".java.policy.backup_<yymmdd>_<hhmmss>.txt"
      // 
      File f = new File( s );
      if (f.exists()) {
        f.renameTo( new File(t) );
      }

      // save current editor contents to .java.policy
      // 
      FileWriter fw = new FileWriter( new File(s) );
      String contents = textArea.getText();
      for( int i=0,n=contents.length(); i < n; i++ ) {
        fw.write( contents.charAt(i) );
      }
      fw.close();
    }
    catch( Exception e ) {

    }
  }


  public static void msg( String s ) {
    System.out.println( s );
  }



  public static void main( String args[] ) {
    JavaPolicyEditor jpe = new JavaPolicyEditor();
    jpe.getJFrame().show();
  }
}
